"use strict";
// Modules
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeOnRequest = void 0;
const debug_1 = __importDefault(require("debug"));
const escapeRegExp = (str) => str.replace(/[.*+?^${}()|[\]\\]/g, "\\$&"); // $& means the whole matched string
/**
 * Event handler for intercepting network requests during tests.
 *
 * @param interceptedRequest Request object
 */
exports.makeOnRequest = (id, failurePattern, buffer = [], apis = []) => {
    return (interceptedRequest) => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b;
        const frame = interceptedRequest.frame();
        const url = interceptedRequest.url();
        debug_1.default("e2e")(url);
        if (/xstate-inspect/.test(url)) {
            return interceptedRequest.continue();
        }
        const matchingApi = apis.find((api) => new RegExp(escapeRegExp(api.path)).test(url));
        matchingApi && debug_1.default("e2e time")("start -> " + matchingApi.path);
        (yield matchingApi) && (frame === null || frame === void 0 ? void 0 : frame.waitForSelector("body"));
        const pathIndex = matchingApi
            ? Number(yield (frame === null || frame === void 0 ? void 0 : frame.evaluate('document.body.getAttribute("data-marionettist-path-index")')))
            : -1;
        const planIndex = matchingApi
            ? Number(yield (frame === null || frame === void 0 ? void 0 : frame.evaluate('document.body.getAttribute("data-marionettist-plan-index")')))
            : -1;
        matchingApi && debug_1.default("e2e time")("start -> " + matchingApi.path);
        const debugReq = (...args) => debug_1.default(`e2e(${id}) ► path(${pathIndex}) ► plan(${planIndex}) ► req(${url})`)(args.join(" ► "));
        matchingApi && debugReq("API", matchingApi.path);
        let outcome = "*";
        if (matchingApi && interceptedRequest.method() !== "OPTIONS") {
            debugReq("PATTERNS", ...failurePattern[planIndex][pathIndex]);
            debugReq("OUTCOMES", ...Object.keys(matchingApi.outcomes || { "*": {} }));
            const outcomeIndex = failurePattern[planIndex][pathIndex].findIndex((outcome) => { var _a; return !!((_a = matchingApi.outcomes) === null || _a === void 0 ? void 0 : _a[outcome]); });
            outcome = failurePattern[planIndex][pathIndex][outcomeIndex] || "*";
            outcomeIndex >= 0 &&
                failurePattern[planIndex][pathIndex].splice(outcomeIndex, 1);
            debugReq("OUTCOME", outcome);
        }
        if (interceptedRequest.method() !== "OPTIONS" &&
            (matchingApi === null || matchingApi === void 0 ? void 0 : matchingApi.deferrals) != null) {
            debugReq("buffer.length", "" + buffer.length);
            const deferrals = buffer.filter((deferred) => matchingApi.deferrals.includes(deferred.id));
            while (deferrals.length > 0) {
                const deferred = deferrals[0];
                if (deferred) {
                    debugReq("buffer.wait", deferred.id);
                    yield deferred;
                    deferrals.shift();
                    debugReq("buffer.resume", deferred.id);
                }
            }
        }
        if (matchingApi) {
            if (interceptedRequest.method() === "OPTIONS") {
                return interceptedRequest.respond({
                    status: 200,
                    headers: {
                        "Access-Control-Allow-Origin": "*",
                        "Access-Control-Allow-Headers": "*",
                    },
                });
            }
            const apiOutcome = ((_a = matchingApi.outcomes) === null || _a === void 0 ? void 0 : _a[outcome]) || ((_b = matchingApi.outcomes) === null || _b === void 0 ? void 0 : _b["*"]);
            if (apiOutcome) {
                if (apiOutcome.status === -1) {
                    debugReq("Aborted");
                    return interceptedRequest.abort();
                }
                const body = JSON.stringify(apiOutcome.body || {});
                debugReq("response", body);
                return interceptedRequest.respond({
                    status: apiOutcome.status || 200,
                    contentType: "application/json",
                    headers: {
                        "Access-Control-Allow-Origin": "*",
                    },
                    body,
                });
            }
            debugReq("response", "{}");
            return interceptedRequest.respond({
                status: 200,
                contentType: "application/json",
                headers: {
                    "Access-Control-Allow-Origin": "*",
                },
                body: JSON.stringify({}),
            });
        }
        debugReq("continue");
        return interceptedRequest.continue();
    });
};
