"use strict";
/**
 * @packageDocumentation
 * @module utils
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.defer = exports.delay = void 0;
/**
 * ```hs
 * delay :: number -> Promise ()
 * ```
 *
 * Creates a promise that resolves after n milliseconds. Defaults to 1s.
 *
 * @param ms Milliseconds to delay the promise by. Defaults to 1s.
 * @returns A promise
 */
exports.delay = (milliseconds = 1000) => new Promise((resolve) => setTimeout(resolve, milliseconds));
exports.defer = (value, id) => {
    let resolve = () => {
        throw Error("Unexpected deferral");
    };
    const promise = new Promise((r) => {
        resolve = () => r(value);
    });
    return Object.assign(promise, { resolve, id });
};
exports.default = exports.delay;
