"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.make = void 0;
const E = __importStar(require("fp-ts/lib/Either"));
const xstate_1 = require("xstate");
const test_1 = require("@xstate/test");
const debug_1 = __importDefault(require("debug"));
const delay_1 = __importStar(require("./delay"));
const api_1 = require("./api");
const xstate_2 = require("xstate");
const decoder_1 = __importDefault(require("./decoder"));
const parseActions = (wrap) => (buffer, debug) => (actions) => (page) => __awaiter(void 0, void 0, void 0, function* () {
    for (let i = 0; i < actions.length; i++) {
        const action = actions[i];
        if (typeof action === "function") {
            debug("pagefunction");
            yield action(page);
            continue;
        }
        debug(action.join(" -> "));
        switch (action[0]) {
            case "delay": {
                yield delay_1.default(action[1]);
                break;
            }
            case "click": {
                const names = Array.isArray(action[1]) ? action[1] : [action[1]];
                yield Promise.all(names.map((_) => page.click(wrap(_))));
                break;
            }
            case "type": {
                yield page.type(wrap(action[1]), `${action[2]}`);
                break;
            }
            case "select": {
                yield page.select(wrap(action[1]), action[2]);
                break;
            }
            case "defer": {
                const names = Array.isArray(action[1]) ? action[1] : [action[1]];
                yield Promise.all(names.map((_) => {
                    const deferred = delay_1.defer(_, _);
                    debug(`deferring -> ${deferred.id}`);
                    buffer.push(deferred);
                    debug(`buffer.length -> ${buffer.length}`);
                }));
                break;
            }
            case "resolve": {
                let index = -1;
                do {
                    index = buffer.findIndex((deferred) => deferred.id === action[1]);
                    if (index >= 0) {
                        const deferred = buffer[index];
                        debug(`resolving -> ${deferred.id}`);
                        buffer.splice(index, 1);
                        deferred.resolve();
                    }
                } while (false);
                break;
            }
            case "waitForSelector": {
                const names = Array.isArray(action[1]) ? action[1] : [action[1]];
                yield Promise.all(names.map((_) => page.waitForSelector(wrap(_))));
                break;
            }
            case "waitForFocus": {
                const fn = `!!document.activeElement && document.activeElement === document.querySelector("${wrap(action[1]).replace(/"/g, '\\"')}")`;
                debug(`waitForFocus -> ${fn}`);
                yield page.waitForFunction(fn);
                break;
            }
            case "expectProperty": {
                const element = yield page.waitForSelector(wrap(action[1]));
                const value = yield page.evaluate((el, selector) => el[selector], element, action[2]);
                expect(value).toBe(action[3]);
                break;
            }
        }
    }
});
const parseActionsToAssign = (actions) => actions
    .map((action) => {
    if (typeof action === "function") {
        return undefined;
    }
    switch (action[0]) {
        case "select":
        case "type": {
            return xstate_2.assign({ [action[1]]: () => action[2] });
        }
        default:
            return undefined;
    }
})
    .filter((_) => !!_);
const defaultSelectorWrapper = (selector) => `[data-testid~="${selector}"]`;
const defaultPorts = {
    ci: 7777,
    prod: 7777,
    dev: 3000,
};
function make({ selectorWrapper = defaultSelectorWrapper, ports: ports_ = defaultPorts, server = "http://localhost", xstateInspect = false, } = {
    selectorWrapper: defaultSelectorWrapper,
    ports: defaultPorts,
    server: "http://localhost",
    xstateInspect: false,
}) {
    const ports = Object.assign(Object.assign({}, defaultPorts), ports_);
    const ci = process.env.CI === "true";
    const prod = process.env.NODE_ENV === "production";
    const port = ci && ports.ci ? ports.ci : ci || prod ? ports.ci || ports.prod : ports.dev;
    return (json) => {
        const configuration = decoder_1.default(json);
        if (E.isRight(configuration)) {
            const config = configuration.right;
            describe(`Auto-generated: ${config.id}`, () => {
                const debugPlan = (...args) => debug_1.default(`e2e(${config.id}): ► PLAN`)(args.join(' ► '));
                const debugPath = (...args) => debug_1.default(`e2e(${config.id}): ► PATH`)(args.join(' ► '));
                const debugTest = (...args) => debug_1.default(`e2e(${config.id}): ► TEST`)(args.join(' ► '));
                const debugEvent = (...args) => debug_1.default(`e2e(${config.id}): ► EVENT`)(args.join(' ► '));
                const machineTemplate = {
                    id: config.id,
                    initial: config.initial,
                    states: {},
                    context: {},
                };
                const eventMap = {};
                Object.keys(config.states).forEach((state) => {
                    const { type, tests, on: events = {}, xstate = {} } = config.states[state];
                    const meta = tests && {
                        test: (page) => __awaiter(this, void 0, void 0, function* () {
                            debugTest(`state: ${state}`);
                            return parseActions(selectorWrapper)(buffer, debugTest)(tests)(page);
                        }),
                    };
                    const on = Object.keys(events || {}).reduce((acc, next) => {
                        const event = events[next];
                        acc[next] = {
                            target: event.target,
                            actions: parseActionsToAssign(event.actions || []),
                        };
                        if (event.actions) {
                            const actions = event.actions;
                            eventMap[next] = {
                                exec: (page) => __awaiter(this, void 0, void 0, function* () {
                                    debugEvent(`${next}`);
                                    return parseActions(selectorWrapper)(buffer, debugEvent)(actions)(page);
                                }),
                            };
                        }
                        else {
                            eventMap[next] = {
                                exec: () => true,
                            };
                        }
                        return acc;
                    }, {});
                    const stateNode = Object.assign({
                        meta,
                        on,
                        type,
                    }, xstate);
                    machineTemplate.states[state] = stateNode;
                });
                const machine = xstate_1.createMachine(machineTemplate);
                const buffer = [];
                const failurePattern = [];
                const onRequest = api_1.makeOnRequest(config.id, failurePattern, buffer, config.apis || []);
                beforeAll(() => __awaiter(this, void 0, void 0, function* () {
                    yield page.setRequestInterception(true);
                    page.on("request", onRequest);
                }));
                afterAll(() => __awaiter(this, void 0, void 0, function* () {
                    page.off("request", onRequest);
                    yield page.setRequestInterception(false);
                }));
                const model = test_1.createModel(machine).withEvents(eventMap);
                const testPlans = model.getShortestPathPlans();
                testPlans.reverse();
                testPlans.forEach((plan, planIndex) => {
                    //
                    failurePattern[planIndex] = [];
                    describe(`${planIndex}: ${plan.description}`, () => {
                        //
                        plan.paths.forEach((path, pathIndex) => {
                            //
                            failurePattern[planIndex][pathIndex] =
                                // path.description.match(/OK|BAD/g) || [];
                                config.outcomes
                                    ? path.description.match(new RegExp(config.outcomes.join("|"), "g")) || []
                                    : [];
                            const outcomes = failurePattern[planIndex][pathIndex];
                            it(`${pathIndex}: (${outcomes.join(", ")}) ${path.description}`, () => __awaiter(this, void 0, void 0, function* () {
                                debugPlan("-------------");
                                debugPlan(planIndex);
                                debugPlan(plan.description);
                                debugPath(pathIndex);
                                debugPath(path.description);
                                debugPath(outcomes.join(", "));
                                if (config.viewport)
                                    yield page.setViewport(config.viewport);
                                if (config.beforVisit) {
                                    yield parseActions(selectorWrapper)(buffer, debugEvent)(config.beforVisit)(page);
                                }
                                const visitPath = /^\//.test(config.visit.path)
                                    ? config.visit.path
                                    : `/${config.visit.path}`;
                                const q = /\?/.test(visitPath) ? "&" : "?";
                                const url = `${server}:${port}${visitPath}${q}xstate-inspect=${xstateInspect}&pathIndex=${pathIndex}&planIndex=${planIndex}&outcomes=${outcomes.join(",")}`;
                                debugPath(url);
                                yield page.goto(url);
                                yield page.waitForSelector('body');
                                yield page.evaluate(`document.body.setAttribute("data-marionettist-path-index", ${pathIndex})`);
                                yield page.evaluate(`document.body.setAttribute("data-marionettist-plan-index", ${planIndex})`);
                                yield path.test(page);
                                while (buffer.length > 0) {
                                    const deferred = buffer.shift();
                                    deferred === null || deferred === void 0 ? void 0 : deferred.resolve();
                                }
                            }), 60000);
                        });
                    });
                });
                it("coverage", () => {
                    model.testCoverage({
                        filter: (node) => !!node.meta,
                    });
                });
            });
        }
        else {
            console.log(JSON.stringify(configuration, null, 2));
            describe(`Failed to generate automated end-2-end test for ${json.name}`, () => {
                it("Failed to decode configuration", () => {
                    expect(false).toBe(configuration.left);
                });
            });
        }
    };
}
exports.make = make;
exports.default = make();
